# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

from typing import List

import torch
from stop_sequencer import StopSequencer
from transformers import AutoModelForCausalLM, AutoTokenizer

from repoqa.provider.base import BaseProvider
from repoqa.provider.request import construct_message_list, hacky_assistant_stop_seq


class HfProvider(BaseProvider):
    def __init__(self, model, trust_remote_code=False, attn_implementation=None):
        self.tokenizer = AutoTokenizer.from_pretrained(
            model, trust_remote_code=trust_remote_code
        )
        self.hf_model = AutoModelForCausalLM.from_pretrained(
            model,
            trust_remote_code=trust_remote_code,
            attn_implementation=attn_implementation,
            torch_dtype="auto",
        ).cuda()
        self.stop_sequencer = StopSequencer(
            self.hf_model,
            model_type="causal",  # or seq2seq
            tokenizer=self.tokenizer,
        )
        self.stop_seq = []
        if self.tokenizer.chat_template:
            self.stop_seq.append(hacky_assistant_stop_seq(self.tokenizer))

    @torch.inference_mode()
    def generate_reply(
        self, question, n=1, max_tokens=1024, temperature=0.0, system_msg=None
    ) -> List[str]:
        assert temperature != 0 or n == 1, "n must be 1 when temperature is 0"

        prompt_tokens = self.tokenizer.apply_chat_template(
            construct_message_list(question, system_msg),
            return_tensors="pt",
            add_generation_prompt=True,
        ).cuda()

        hf_model = self.hf_model
        input_length = prompt_tokens.size(-1)
        if self.stop_seq:
            hf_model = self.stop_sequencer.register_stop_texts(
                stop_texts=self.stop_seq,
                input_length=input_length,
            )

        gen_args = {"do_sample": False}
        if temperature > 0:
            gen_args["do_sample"] = True
            gen_args["temperature"] = temperature

        output_text = hf_model.generate(
            input_ids=prompt_tokens,
            max_new_tokens=max_tokens,
            num_return_sequences=n,
            pad_token_id=self.tokenizer.eos_token_id,
            use_cache=True,
            **gen_args,
        )

        gen_strs = [
            self.tokenizer.decode(
                x[input_length:],
                skip_special_tokens=True,
                clean_up_tokenization_spaces=False,
            )
            for x in output_text
        ]
        return gen_strs



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
import os

import openai
from tqdm import tqdm

from repoqa.provider.request.openai import make_auto_request
from repoqa.utility import topological_sort

CAPTURE_HEAD = "<desc_start>"
CAPTURE_TAIL = "<desc_end>"


def make_prompt(fn_name: str, code: str):
    instruction = f'Can you **briefly** describe the purpose, input, output, and procedure of "{fn_name}"?'
    return f"""\
{instruction}

```
{code}
```

{instruction}

Please follow format to complete the skeleton below:

{CAPTURE_HEAD}
1. **Purpose**: ...
2. **Input**: ...
3. **Output**: ...
4. **Procedure**: ...
{CAPTURE_TAIL}

{instruction}

Notes:
1. DO NOT reveal function names ({fn_name}) and variable names
2. Start with {CAPTURE_HEAD} and end with {CAPTURE_TAIL}
3. Customize the description to differentiate it from other functions
"""


# Annotate an incomplete repoqa dataset with function and class information
def main(
    dataset_path: str,
    code_prefix_lines: int = 100,
    output_desc_path: str = "function_description.jsonl",
    use_batch_api: bool = False,
    verbose: bool = False,
    debug: bool = False,
):
    assert use_batch_api == False, "Batch API is not supported yet."

    assert dataset_path.endswith(".json"), "Dataset must be a JSON file, check README"
    with open(dataset_path, "r") as f:
        lists = json.load(f)

    # resume from output_desc_path
    if output_desc_path.endswith(".jsonl") and os.path.exists(output_desc_path):
        with open(output_desc_path, "r") as f:
            results = [json.loads(line) for line in f]
    else:
        # {repo, name, prompt, annotation}
        results = []

    # a set of inference task to run; each item is a tuple of {repo, name, prompt}
    tasks = []
    for lang, repos in lists.items():
        print(f"🔥 Collecting unannotated needle functions for {lang}")
        for repo in tqdm(repos):
            if not repo.get("dependency"):
                print(
                    f"⚠️ Skipping {repo['repo']} ({lang}) as it does not have `dependency` -- do dependency analysis first"
                )
                continue
            ordered_paths = topological_sort(repo["dependency"])
            repo_lines = []
            for path in ordered_paths:
                repo_lines.extend(repo["content"][path].split("\n"))

            def get_code(global_start_line, global_end_line):
                return "\n".join(
                    repo_lines[
                        max(0, global_start_line - code_prefix_lines) : global_end_line
                    ]
                )

            existing_needles = set(
                [item["name"] for item in results if item["repo"] == repo["repo"]]
            )

            for needle in repo["needles"]:
                fn_name = needle["name"]
                if fn_name in existing_needles:
                    continue
                code = get_code(needle["global_start_line"], needle["global_end_line"])
                prompt = make_prompt(fn_name, code)
                if verbose:
                    print(prompt)
                    print("-" * 80)
                tasks.append(
                    {
                        "repo": repo["repo"],
                        "name": fn_name,
                        "prompt": prompt,
                    }
                )

    print(f"🔥 {len(tasks)} needle functions to be annotated in total")
    client = openai.Client()
    with open(output_desc_path, "+a") as f_out:
        for task in tqdm(tasks):
            print(f"🔥 Annotating {task['name']} in {task['repo']}")
            output = make_auto_request(
                client,
                task["prompt"],
                model="gpt-4-turbo",
                max_tokens=2048,
                temperature=0.2,
                n=1,
            )
            annotation = output.choices[0].message.content
            result = {
                "repo": task["repo"],
                "name": task["name"],
                "prompt": task["prompt"],
                "raw_annotation": annotation,
                "annotation": annotation.split(CAPTURE_HEAD)[-1].split(CAPTURE_TAIL)[0],
            }
            json.dump(result, f_out)
            f_out.write("\n")
            f_out.flush()

            if debug:
                print("[PROMPT]", "-" * 80)
                print(task["prompt"])
                print("[ANNOTATION]", "-" * 80)
                print(annotation)
                print("-" * 80)
                print("Enter to continue... or b to break:")
                if input() == "b":
                    break


if __name__ == "__main__":
    from fire import Fire

    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0


def main(model="deepseek-ai/deepseek-coder-6.7b-instruct", max_tokens=8 * 2**10):
    import time

    import openai

    client = openai.OpenAI(  # Note: if you need UIUC VPN or UIUC network to access the server!
        api_key="none", base_url="http://ise-dynamo.cs.illinois.edu:8888/v1"
    )

    prompt = "def "

    tstart = time.time()
    responses = client.completions.create(
        model=model, prompt=prompt, n=1, max_tokens=max_tokens, temperature=0
    )
    print(f"Time taken: {time.time() - tstart:.1f}s")
    print("Finish reason:", responses.choices[0].finish_reason)
    print(
        "Estimated max context length in #chars is: ",
        len(prompt) + len(responses.choices[0].text),
    )


if __name__ == "__main__":
    from fire import Fire

    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
import os
import subprocess

import git
import tempdir
from fire import Fire
from tqdm.auto import tqdm

from scripts.curate.dataset_ensemble_clone import get_files_to_include
from scripts.curate.utility import lang2suffix

CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))


# dataset_path is the dataset generated by dataset_ensemble_clone.py
def main():
    with open("scripts/cherrypick/lists.json") as f:
        lists = json.load(f)

    lang_suffix = lang2suffix["python"]
    repos = lists["python"]
    for repo in tqdm(repos):
        repo_name = repo["repo"]
        commit_sha = repo["commit_sha"]
        entrypoint = repo["entrypoint_path"]
        print(f"Visiting https://github.com/{repo_name}/tree/{commit_sha}")

        with tempdir.TempDir() as temp_dir:
            gh_repo = git.Repo.clone_from(
                f"https://github.com/{repo_name}.git",
                temp_dir,
            )
            gh_repo.git.checkout(commit_sha)
            command_list = f"pydeps {entrypoint} --show-deps --no-show".split()
            # cd `temp_dir`` and capture the output json
            output = subprocess.check_output(command_list, cwd=temp_dir)
            dependencies = json.loads(output)

            output_dependency = {}
            mod2path = {}

            abs_prefix = os.path.join(temp_dir, entrypoint)
            for mod, v in dependencies.items():
                if v["path"] and v["path"].startswith(abs_prefix):
                    mod2path[mod] = v["path"]

            for mod, v in dependencies.items():
                if v["path"] and v["path"].startswith(abs_prefix):
                    if "imports" in v:
                        relative_path = os.path.relpath(v["path"], temp_dir)
                        output_dependency[relative_path] = [
                            os.path.relpath(mod2path[imp], temp_dir)
                            for imp in v["imports"]
                            if imp in mod2path
                        ]

            files_to_include = get_files_to_include(gh_repo, entrypoint, lang_suffix)
            for path, _ in files_to_include:
                if path not in output_dependency:
                    output_dependency[path] = []

            assert output_dependency, "Empty output_dependency"
            repo["dependency"] = output_dependency

    with open(os.path.join(CURRENT_DIR, "data", "python.json"), "w") as f_out:
        json.dump({"python": repos}, f_out)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
import os
from datetime import datetime
from typing import TypedDict
from zoneinfo import ZoneInfo

from fire import Fire
from github import Auth, Github
from tqdm.auto import tqdm

from scripts.curate.utility import lang2suffix


class GitHubRepoMeta(TypedDict):
    repo_name: str
    repo_owner: str
    commit_sha: str
    repo_size: int


class GitHubDocument(GitHubRepoMeta):
    timestamp: str
    path: str
    content: str


def main(
    language: str = "python",
    stars: int = 100,
    minimal_new_commits: int = 50,
    new_commit_since: str = "2024-01-01",
    minimal_lang_bytes: int = 1024 * 64,  # 64k ideally
):
    token = os.getenv("GITHUB_TOKEN")
    assert token is not None, "Make a token at https://github.com/settings/tokens"
    auth = Auth.Token(token)

    # See repo selection criteria at https://github.com/evalplus/repoqa/issues/1
    query = []
    query.append(f"language:{language}")
    query.append(f"stars:>={stars}")
    query.append("license:mit license:apache-2.0")
    query.append(f"pushed:>={new_commit_since}")
    # 128KB to 32MB
    query.append("size:128..32000")
    query.append("sort:stars")

    # compile query
    query = " ".join(query)
    print(f"{query=}")
    g = Github(auth=auth, per_page=100)

    lang_suffix = lang2suffix[language]

    with open(f"{language}-{datetime.now().isoformat()}.jsonl", "w") as f_out:
        repos = g.search_repositories(query)
        print("Found ", repos.totalCount, "repositories for", language)
        for repo in tqdm(repos, total=repos.totalCount):
            # filter at least 100 commits have been made since 2023 Q4 (>= 2023-09-01).
            commits = repo.get_commits()
            if (
                count_ := sum(
                    True
                    for commit in commits
                    if commit.last_modified_datetime
                    >= datetime.strptime(new_commit_since, "%Y-%m-%d").replace(
                        tzinfo=ZoneInfo("UTC")
                    )
                )
            ) < minimal_new_commits:
                print(
                    f"Skip {repo.html_url} for have less than {minimal_new_commits} after {new_commit_since} (only {count_} commits)"
                )
                continue

            # filter repos that is large enough
            git_tree = repo.get_git_tree(repo.default_branch, recursive=True)

            tree_iter = list(
                filter(
                    lambda item: item.type == "blob"
                    and any([item.path.endswith(suffix) for suffix in lang_suffix]),
                    tqdm(git_tree.tree, leave=False),
                )
            )

            code_file_size = int(sum(item.size for item in tree_iter))
            if code_file_size < minimal_lang_bytes:
                print(
                    f"Skip {repo.html_url} for have less than {minimal_lang_bytes} bytes source file after {new_commit_since} (only {code_file_size} bytes)"
                )
                continue

            schema = dict(
                repo_name=repo.name,
                repo_size=code_file_size,
                repo_owner=repo.owner.login,
                repo_url=repo.html_url,
                commit_sha=git_tree.sha,
                last_modified_time=git_tree.last_modified_datetime,
                content={},
            )
            for item in tree_iter:
                # Fetch the content for each Python file
                content = repo.get_contents(item.path)
                assert not isinstance(content, list)
                if content.encoding != "base64":
                    continue
                file_content = content.decoded_content.decode("utf-8")
                schema["content"][item.path] = file_content
            f_out.write(json.dumps(schema) + "\n")
            f_out.flush()


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
import os
import re
import subprocess
from pathlib import Path
from typing import Dict, List

import git
import tempdir
from fire import Fire
from tqdm.auto import tqdm

from scripts.curate.dataset_ensemble_clone import get_files_to_include
from scripts.curate.utility import lang2suffix

CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))


def traverse_dep(
    current_tree: Dict, current_file: str, entrypoint: str, dependencies: Dict
) -> None:
    if current_tree == None:
        dependencies[current_file] = set()
        return

    results = set(
        [
            file
            for file in current_tree.keys()
            if entrypoint in file and Path(file).suffix in [".ts", ".js"]
        ]
    )
    dependencies[current_file] = results

    for result in results:
        traverse_dep(current_tree[result], result, entrypoint, dependencies)


def add_circular(circular_dependencies: Dict, dependencies: Dict):
    for cycle in circular_dependencies:
        for index, from_file in enumerate(cycle[:-1]):
            to_file = cycle[index + 1]
            dependencies[from_file].add(to_file)


def get_dependencies(
    file_path: Path, temp_dir: str, entrypoint: str, dependencies: Dict
) -> None:
    command_list = f"dep-tree tree {file_path} --json".split()
    output_string = subprocess.check_output(command_list, cwd=temp_dir)
    output_string = output_string.decode("utf-8")[1:-2].split()
    json_string = "".join(chr(int(value)) for value in output_string)

    json_data = json.loads(json_string)
    tree_data = json_data["tree"]
    circular_dependencies = json_data["circularDependencies"]
    current_file = list(tree_data.keys())[0]

    traverse_dep(tree_data[current_file], current_file, entrypoint, dependencies)
    add_circular(circular_dependencies, dependencies)


# dataset_path is the dataset generated by dataset_ensemble_clone.py
def main():
    with open("scripts/cherrypick/lists.json") as f:
        lists = json.load(f)

    lang_suffix = lang2suffix["typescript"]
    repos = lists["typescript"]
    for repo in tqdm(repos):
        repo_name = repo["repo"]
        commit_sha = repo["commit_sha"]
        entrypoint = repo["entrypoint_path"]
        print(f"Visiting https://github.com/{repo_name}/tree/{commit_sha}")

        dependencies = {}
        with tempdir.TempDir() as temp_dir:
            gh_repo = git.Repo.clone_from(
                f"https://github.com/{repo_name}.git",
                temp_dir,
            )
            gh_repo.git.checkout(commit_sha)
            abs_prefix = Path(os.path.join(temp_dir, entrypoint))
            if "/" in entrypoint:
                suffix_path = entrypoint.split("/")[-1]
            else:
                suffix_path = entrypoint
            for file_path in abs_prefix.rglob("*"):
                if file_path.is_file() and file_path.suffix in lang_suffix:
                    current_name = os.path.relpath(str(file_path), temp_dir)
                    if current_name in dependencies:
                        continue
                    get_dependencies(file_path, temp_dir, suffix_path, dependencies)

            for key, value in dependencies.items():
                dependencies[key] = list(value)

            if "/" in entrypoint:
                updated_dependencies = {}
                append_prefix = "/".join(entrypoint.split("/")[:-1]) + "/"
                for key, values in dependencies.items():
                    new_key = append_prefix + key
                    if not ((Path(temp_dir) / Path(new_key)).exists()):
                        continue
                    new_values = []
                    for value in values:
                        new_value = append_prefix + value
                        if (Path(temp_dir) / Path(new_value)).exists():
                            new_values.append(new_value)
                    updated_dependencies[new_key] = new_values
                dependencies = updated_dependencies
            repo["dependency"] = dependencies
    with open(os.path.join(CURRENT_DIR, "data", "typescript.json"), "w") as f_out:
        json.dump({"typescript": repos}, f_out)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import os
from typing import List

import google.generativeai as genai

from repoqa.provider.base import BaseProvider
from repoqa.provider.request.google import make_auto_request


class GoogleProvider(BaseProvider):
    def __init__(self, model):
        genai.configure(api_key=os.getenv("GOOGLE_API_KEY"))
        self.model = model
        self.client = genai.GenerativeModel(model)

    def generate_reply(
        self, question, n=1, max_tokens=1024, temperature=0.0, system_msg=None
    ) -> List[str]:
        assert temperature != 0 or n == 1, "n must be 1 when temperature is 0"
        replies = make_auto_request(
            self.client,
            question,
            self.model,
            n=n,
            max_tokens=max_tokens,
            temperature=temperature,
            system_msg=system_msg,
        )

        if len(replies.candidates) != n:
            print(f"[WARNING] # replies = {len(replies.candidates)} != {n = }")

        ret_texts = []
        for candidate in replies.candidates:
            parts = candidate.content.parts
            if parts:
                ret_texts.append(parts[0].text)
            else:
                print("Empty response!")
                ret_texts.append("")
                print(f"{candidate.safety_ratings = }")

        return ret_texts + [""] * (n - len(ret_texts))



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

if __name__ == "__main__":
    import openai

    client = openai.OpenAI(  # Note: if you need UIUC VPN or UIUC network to access the server!
        api_key="none", base_url="http://ise-dynamo.cs.illinois.edu:8888/v1"
    )

    task_prefix = "def fibonacci(n):\n"
    prompt = f"""This is the fastest implementation for Fibonacci:
```python
{task_prefix}"""

    # completion
    responses = client.completions.create(
        model="deepseek-ai/deepseek-coder-6.7b-instruct",
        prompt=prompt,
        max_tokens=256,
        n=3,
        stop=["\n```"],
    )

    for c in responses.choices:
        print(task_prefix + c.text)
        print("=" * 8)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import os
from typing import List

from openai import Client
from transformers import AutoTokenizer

from repoqa.provider.base import BaseProvider
from repoqa.provider.request import hacky_assistant_stop_seq
from repoqa.provider.request.openai import make_auto_request


class OpenAIProvider(BaseProvider):
    def __init__(self, model, base_url: str = None):
        self.model = model
        self.client = Client(
            api_key=os.getenv("OPENAI_API_KEY", "none"), base_url=base_url
        )
        self.stop_seq = []
        try:
            tokenizer = AutoTokenizer.from_pretrained(model)
            if tokenizer.chat_template:
                self.stop_seq.append(hacky_assistant_stop_seq(tokenizer))
            print("Using stop sequence: ", self.stop_seq)
        except:
            print("Failed to automatically fetch stop tokens from HuggingFace.")

    def generate_reply(
        self, question, n=1, max_tokens=1024, temperature=0.0, system_msg=None
    ) -> List[str]:
        assert temperature != 0 or n == 1, "n must be 1 when temperature is 0"
        replies = make_auto_request(
            self.client,
            message=question,
            model=self.model,
            temperature=temperature,
            n=n,
            max_tokens=max_tokens,
            system_msg=system_msg,
            stop=self.stop_seq,
        )

        return [reply.message.content for reply in replies.choices]



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
import os

from fire import Fire


def main(dataset_path: str):
    # iterate json files under scripts/curate/dep_analysis/data
    repo2dep = {}
    for file in os.listdir(os.path.join("scripts/curate/dep_analysis/data")):
        if file.endswith(".json"):
            with open(os.path.join("scripts/curate/dep_analysis/data", file)) as f:
                data = json.load(f)

            repos = list(data.values())[0]
            for repo in repos:
                repo2dep[repo["repo"]] = repo["dependency"]

    with open(dataset_path) as f:
        dataset = json.load(f)

    for lang, repos in dataset.items():
        for repo in repos:
            if repo["repo"] not in repo2dep:
                print(f"{lang} -- Repo {repo['repo']} not found in dep analysis data")
                continue
            repo["dependency"] = repo2dep[repo["repo"]]
            print(f"{lang} -- Repo {repo['repo']} has dependency added in the dataset")

    with open(dataset_path, "w") as f_out:
        json.dump(dataset, f_out)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
import os
from pathlib import Path
from typing import List, Union

import git
import tempdir
from fire import Fire
from tqdm.auto import tqdm

from scripts.curate.utility import lang2suffix

POTENTIAL_INCLUDE_DIR_NAMES = [
    "include",
    "src",
    "lib",
    "libs",
    "library",
    "libraries",
    "inc",
]

CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))


def get_potential_include_dirs(temp_dir: str) -> List[Path]:
    include_dir_list = [Path(temp_dir)]
    for potential_include_dir_name in POTENTIAL_INCLUDE_DIR_NAMES:
        for dir in Path(temp_dir).rglob(potential_include_dir_name):
            include_dir_list.append(dir)

    return include_dir_list


def search_include(
    entrypoint: str,
    temp_dir: str,
    cpp_file: Path,
    include: str,
    additional_libs: List[Path] = [],
) -> Union[Path, None]:
    for possible_path in [cpp_file.parent] + additional_libs:
        if not (entrypoint in str(possible_path)):
            continue
        if (possible_path / include).exists():
            return (
                (possible_path / include)
                .resolve()
                .relative_to(Path(temp_dir))
                .as_posix()
            )


def get_dependencies(temp_dir: str, entrypoint: str, cpp_file: Path):
    user_defined_includes = []
    with open(cpp_file) as f:
        content = f.read()
        for line in content.split("\n"):
            line = line.strip()
            if line.startswith("#include"):
                include = line.split()[1]
            elif line.split()[:2] == ["#", "include"]:
                include = line.split()[2]
            else:
                continue

            if (include.startswith('"') or include.startswith("<")) and "." in include:
                include = include[1:-1]
                rela_include = search_include(
                    entrypoint,
                    temp_dir,
                    cpp_file,
                    include,
                    additional_libs=get_potential_include_dirs(temp_dir),
                )
                if rela_include:
                    user_defined_includes.append(rela_include)

    return user_defined_includes


# dataset_path is the dataset generated by dataset_ensemble_clone.py
def main():
    with open("scripts/cherrypick/lists.json") as f:
        lists = json.load(f)

    lang_suffix = lang2suffix["cpp"]
    repos = lists["cpp"]
    for repo in tqdm(repos):
        repo_name = repo["repo"]
        commit_sha = repo["commit_sha"]
        entrypoint = repo["entrypoint_path"]

        print(f"Visiting https://github.com/{repo_name}/tree/{commit_sha}")

        if repo.get("dependency"):
            print(f"Skipping {repo_name} as it already has dependency field.")
            continue

        with tempdir.TempDir() as temp_dir:
            gh_repo = git.Repo.clone_from(
                f"https://github.com/{repo_name}.git",
                temp_dir,
            )
            gh_repo.git.checkout(commit_sha)

            output_dependency = {}
            abs_prefix = os.path.join(temp_dir, entrypoint)
            for cpp_ext in lang_suffix:
                for cpp_file in Path(abs_prefix).rglob("*" + cpp_ext):
                    dependencies = get_dependencies(temp_dir, entrypoint, cpp_file)
                    if len(dependencies) > 0:
                        if "" in dependencies:
                            dependencies.remove("")
                        output_dependency[
                            cpp_file.relative_to(Path(temp_dir)).as_posix()
                        ] = dependencies
                    else:
                        output_dependency[
                            cpp_file.relative_to(Path(temp_dir)).as_posix()
                        ] = []

            repo["dependency"] = output_dependency

    with open(os.path.join(CURRENT_DIR, "data", "cpp.json"), "w") as f_out:
        json.dump({"cpp": repos}, f_out)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import itertools
import json
import os
import re
from collections import defaultdict
from datetime import datetime
from enum import Enum
from pathlib import Path
from typing import Dict, List, Tuple, Union

import numpy as np
import tempdir
from rich.console import Console
from rich.table import Table
from transformers import AutoConfig
from tree_sitter_languages import get_language, get_parser

from repoqa.data import get_repoqa_data
from repoqa.metric import compute_function_similarity
from repoqa.utility import COMMENT_QUERY, FUNCTION_QUERY, progress

LANGUAGES = list(FUNCTION_QUERY.keys())
THRESHOLDS = [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]


class Result(Enum):
    BEST_MATCH = "best_match"
    FAIL_MATCH = "fail_match"


# unbiased estimator from https://github.com/openai/human-eval
def estimate_pass_at_k(
    num_samples: Union[int, List[int], np.ndarray],
    num_correct: Union[List[int], np.ndarray],
    k: int,
) -> np.ndarray:
    """
    Estimates pass@k of each problem and returns them in an array.
    """

    def estimator(n: int, c: int, k: int) -> float:
        """
        Calculates 1 - comb(n - c, k) / comb(n, k).
        """
        if n - c < k:
            return 1.0
        return 1.0 - np.prod(1.0 - k / np.arange(n - c + 1, n + 1))

    if isinstance(num_samples, int):
        num_samples_it = itertools.repeat(num_samples, len(num_correct))
    else:
        assert len(num_samples) == len(num_correct)
        num_samples_it = iter(num_samples)

    return np.array(
        [estimator(int(n), int(c), k) for n, c in zip(num_samples_it, num_correct)]
    )


def remove_comments(source_code: str, lang: str) -> str:
    source_bytes = bytes(source_code, "utf8")
    parser = get_parser(lang)
    tree = parser.parse(source_bytes)
    root_node = tree.root_node

    # Remove comments from source code
    capture_list = []
    for query_str in COMMENT_QUERY[lang]:
        comment_query = get_language(lang).query(query_str)
        capture_list += comment_query.captures(root_node)

    capture_list.sort(key=lambda cap: cap[0].start_byte, reverse=True)

    for node, _ in capture_list:
        source_bytes = source_bytes[: node.start_byte] + source_bytes[node.end_byte :]

    return source_bytes.decode("utf-8")


def sanitize_output(model_output: str, lang: str) -> str:
    model_output = model_output.strip()
    search_pattern = r"^```(?:\w+)?\s*\n(.*?)(?=^```)```"
    code_blocks = re.findall(search_pattern, model_output, re.DOTALL | re.MULTILINE)

    parser = get_parser(lang)
    fn_query = get_language(lang).query(FUNCTION_QUERY[lang])

    # If not code blocks found, simply return model output
    if not code_blocks:
        return model_output

    processed_blocks = []
    for block in code_blocks:
        processed_blocks.append(block)

        # Try to use tree-sitter to parse if possible
        try:
            block_bytes = bytes(block, "utf8")
            tree = parser.parse(block_bytes)
            for capture in fn_query.captures(tree.root_node):
                node, _ = capture
                function_content = block_bytes[node.start_byte : node.end_byte]
                return function_content.decode("utf8")
        except:
            pass

    # no valid functions found by tree-sitter approach return first block
    return processed_blocks[0]


def print_result_table(model_name, pass_results):
    # Printing scores in a table
    table = Table(title=f"Scores (%) of {model_name} at different thresholds")
    table.add_column("Threshold", justify="center", style="bold magenta")
    for threshold in THRESHOLDS:
        table.add_column(f"{threshold}", justify="center")

    # Prepare data to determine the maximum values for each threshold
    threshold_scores = {threshold: [] for threshold in THRESHOLDS}
    for lang_results in pass_results.values():
        for thresh, value in lang_results.items():
            threshold_scores[thresh].append(value["pass@1"])

    # Calculate the maximum score for each threshold
    max_scores = {
        threshold: max(scores) for threshold, scores in threshold_scores.items()
    }
    min_scores = {
        threshold: min(scores) for threshold, scores in threshold_scores.items()
    }

    # Fill the table rows
    for language, lang_results in pass_results.items():
        row = [("⭐" if language == "all" else "") + language]
        for threshold, value in lang_results.items():
            score = value["pass@1"]
            formatted_score = f"{100 * score:.1f}"
            if max_scores[threshold] - score < 0.01:
                formatted_score = f"[bold green]{formatted_score}[/]"
            elif score - min_scores[threshold] < 0.01:
                formatted_score = f"[bold red]{formatted_score}[/]"
            row.append(formatted_score)
        if language == "all":
            row = [f"[bold yellow]{r}[/]" for r in row]
        table.add_row(*row)

    Console().print(table)


def needle_evaluator(
    model_output: str,
    ground_truth: str,
    repo_info: Dict,
    lang: str,
    ignore_comments: bool,
) -> Tuple[Result, str, float]:
    contents = repo_info["content"]
    needles = repo_info["needles"]

    best_target = None
    best_similarity = 0
    sanitized_output = sanitize_output(model_output, lang)
    if ignore_comments:
        sanitized_output = remove_comments(sanitized_output, lang)
    for needle in needles:
        current_path = needle["path"]
        current_name = needle["name"]
        current_func = "\n".join(
            contents[current_path].split("\n")[
                needle["start_line"] : needle["end_line"]
            ]
        )
        if ignore_comments:
            current_func = remove_comments(current_func, lang)

        current_similarity = compute_function_similarity(sanitized_output, current_func)
        if current_similarity > best_similarity:
            best_similarity = current_similarity
            best_target = current_name

    if best_target == ground_truth:
        verdict = Result.BEST_MATCH
    else:
        verdict = Result.FAIL_MATCH
    return verdict, best_target, best_similarity


def _get_repo(lang_data: Dict, repo_name: str) -> Dict:
    for repo in lang_data:
        if repo["repo"] == repo_name:
            return repo


def compute_language_results(evaluation_result: Dict, all_results: Dict) -> None:
    for language, lang_results in evaluation_result.items():
        current_result = {}
        total = np.array([1 for _ in lang_results])

        for threshold in THRESHOLDS:
            correct_result = []
            for res in lang_results:
                bc = 0
                if res["is_best_similar"] and res["best_similar_score"] >= threshold:
                    bc = 1
                correct_result.append(bc)
            correct_result = np.array(correct_result)

            pass_at_k = {
                f"pass@{k}": estimate_pass_at_k(total, correct_result, k).mean()
                for k in [1, 10, 100]
                if total.min() >= k
            }
            current_result[threshold] = pass_at_k
        all_results[language] = current_result


def fetch_hf_context(model_name: str) -> str:
    # Retrieved from https://github.com/vllm-project/vllm/blob/main/vllm/config.py#L1073
    possible_keys = [
        # OPT
        "max_position_embeddings",
        # GPT-2
        "n_positions",
        # MPT
        "max_seq_len",
        # ChatGLM2
        "seq_length",
        # Command-R
        "model_max_length",
        # Others
        "max_sequence_length",
        "max_seq_length",
        "seq_len",
    ]
    try:
        with tempdir.TempDir() as temp_dir:
            config = AutoConfig.from_pretrained(
                model_name,
                cache_dir=temp_dir,
                force_download=True,
                trust_remote_code=True,
            ).to_dict()
            longest_context = 0
            for key in possible_keys:
                if key in config:
                    longest_context = max(config[key], longest_context)
            if not (longest_context):
                return "N/A"
            return str(int(longest_context / 1024)) + "k"
    except Exception as err:
        print(f"fetching failed... Reason:\n{err}")
        return "N/A"


def compute_score(
    model_name: str, dataset: Dict, model_output: List[Dict], ignore_comments: bool
) -> Dict:
    evaluation_result = defaultdict(list)
    with progress(f"Scoring {model_name}") as pbar:
        for result in pbar.track(model_output):
            lang = result["language"]
            repo_name = result["repo"]
            model_outputs = result["output"]
            ground_truth = result["name"]
            repo_info = _get_repo(dataset[lang], repo_name)

            model_output = model_outputs[0]
            verdict, best_target, best_similarity = needle_evaluator(
                model_output, ground_truth, repo_info, lang, ignore_comments
            )

            is_best_similar = False
            if verdict == Result.BEST_MATCH:
                is_best_similar = True

            current_task = {
                "repo": repo_name,
                "name": ground_truth,
                "needle_position": result["position_ratio"],
                "is_best_similar": is_best_similar,
                "best_similar_score": best_similarity,
                "best_target": best_target,
                "position": {
                    "token_start": result["needle_token_start"],
                    "token_end": result["needle_token_end"],
                },
            }
            evaluation_result[lang].append(current_task)

    # Calculate pass@k
    pass_results = {}

    all_langs = []
    for lang in evaluation_result:
        all_langs += evaluation_result[lang]
    total = np.array([1 for _ in all_langs])

    pass_results["all"] = {}
    for threshold in THRESHOLDS:
        correct_result = []
        for res in all_langs:
            bc = 0
            if res["is_best_similar"] and res["best_similar_score"] >= threshold:
                bc = 1
            correct_result.append(bc)
        correct_result = np.array(correct_result)
        pass_at_k = {
            f"pass@{k}": estimate_pass_at_k(total, correct_result, k).mean()
            for k in [1, 10, 100]
            if total.min() >= k
        }
        pass_results["all"][threshold] = pass_at_k

    compute_language_results(evaluation_result, pass_results)
    print_result_table(model_name, pass_results)

    output_json = {}
    model_json = {}
    model_json["eval_date"] = str(datetime.now())

    # hardcode paid models
    if "/" in model_name:
        if model_name.startswith("bigcode/starcoder2"):
            train_context = "16k"
        else:
            train_context = fetch_hf_context(model_name)
    elif model_name.startswith("gpt-4-turbo") or model_name.startswith("gpt-4o-"):
        train_context = "128k"
    elif model_name.startswith("gpt-3.5-"):
        train_context = "16k"
    elif model_name.startswith("gemini-1.5-pro") or model_name.startswith(
        "gemini-1.5-flash"
    ):
        train_context = "1000k"
    elif model_name.startswith("gemini-1.0-pro"):
        train_context = "32k"
    elif model_name.startswith("claude-3-"):
        train_context = "200k"
    else:
        train_context = "N/A"
    model_json["train_size"] = train_context
    model_json["scores"] = pass_results
    model_json["results"] = evaluation_result

    output_json[model_name] = model_json

    return output_json


def get_model_name(output_path: str) -> str:
    file_name = Path(output_path).stem
    segments = file_name.split("_")
    output_name = ""
    for segment in segments:
        if segment == "slash":
            output_name += "/"
        else:
            output_name += segment
    return output_name


def save_json(output_json, result_path) -> None:
    if os.path.isfile(result_path):
        decision = ""
        while decision.lower() not in ["y", "n"]:
            print(f"{result_path} already exists. Press [Y/N] to overwrite or exit...")
            decision = input()

        if decision.lower() == "y":
            # mv the file to a backup
            new_path = result_path + ".bak"
            while os.path.isfile(new_path):
                new_path += ".bak"
            os.rename(result_path, new_path)
            print(f"Backup {result_path} to {new_path}")

    if not os.path.isfile(result_path):
        with open(result_path, "w") as f:
            json.dump(output_json, f)


def compute_main(
    model_output_path: str, ignore_comments: bool = False, dataset_path: str = None
):
    if dataset_path is None:
        dataset = get_repoqa_data()
    else:
        with open(dataset_path, "r") as dataset_f:
            dataset = json.load(dataset_f)

    model_outputs = []
    with open(model_output_path, "r") as output_f:
        for line in output_f:
            model_outputs.append(json.loads(line))

    file_base, _ = os.path.splitext(model_output_path)
    result_path = file_base + "-SCORES.json"
    model_name = get_model_name(model_output_path)
    output_json = compute_score(model_name, dataset, model_outputs, ignore_comments)
    save_json(output_json, result_path)


def main():
    from fire import Fire

    Fire(compute_main)


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0


def construct_message_list(message, system_message=None):
    msglist = [{"role": "user", "content": message}]
    if system_message:
        msglist.insert(0, {"role": "system", "content": system_message})
    return msglist


def hacky_assistant_stop_seq(tokenizer) -> str:
    _magic_string_ = "&==NowOrNever==&Accelerate!!!==&"
    return tokenizer.apply_chat_template(
        [
            {"role": "user", "content": ""},
            {"role": "assistant", "content": _magic_string_},
        ],
        tokenize=False,
    ).split(_magic_string_)[-1]



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

from abc import ABC, abstractmethod
from typing import List


class BaseProvider(ABC):
    @abstractmethod
    def generate_reply(
        self, question, n=1, max_tokens=1024, temperature=0.0, system_msg=None
    ) -> List[str]:
        ...



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

# Run with ```python scripts/curate/obfuscate_nl.py repoqa-2024-04-20.json```
# Will save to repoqa-2024-04-20-obfuscated.json

import json
import os
import re

from fire import Fire
from tqdm import tqdm
from tree_sitter_languages import get_language, get_parser

from repoqa.utility import COMMENT_QUERY, FUNCTION_NAME_QUERY


def remove_comments(code, language):
    query_texts = COMMENT_QUERY[language]
    parser = get_parser(language)
    code_bytes = bytes(code, "utf8")
    tree = parser.parse(code_bytes)
    comment_chunks = []
    for query_text in query_texts:
        comment_query = get_language(language).query(query_text)
        for node, _ in comment_query.captures(tree.root_node):
            comment_chunks.append(node.text.decode("utf-8"))
    comment_chunks.sort(key=len, reverse=True)
    for chunk in comment_chunks:
        chunk_lines = chunk.splitlines()
        chunk_lines_len = [len(bytes(line, "utf-8")) for line in chunk_lines]
        chunk_lines_empty = [
            (bytes("", "utf-8").ljust(llen, b"\0")).decode("utf-8")
            for llen in chunk_lines_len
        ]
        chunk_empty = "\0".join(chunk_lines_empty)
        chunk_empty = chunk_empty[:-1] + "\n"
        code = code.replace(chunk, chunk_empty)
    return code


def rename_functions(code, language, starting_index=0):
    func_name_query = get_language(language).query(FUNCTION_NAME_QUERY[language])
    parser = get_parser(language)
    print(f"Running rename_functions: {code}, {language}")
    code_bytes = bytes(code, "utf8")
    tree = parser.parse(code_bytes)
    function_names = set()
    for capture in func_name_query.captures(tree.root_node):
        node, _ = capture
        function_names.add(node.text.decode("utf-8"))
    function_map = {}
    current_index = starting_index
    for name in function_names:
        function_map[name] = f"function_{starting_index}"
        code = code.replace(name, function_map[name])
        current_index += 1
    return code, function_map


def main(ds_filepath: str):
    dataset_file = open(ds_filepath, "r")
    dataset = dataset_file.read()
    dataset = json.loads(dataset)
    dataset_file.close()

    for lang in dataset.keys():
        print(f"🔥 Processing language: {lang}")
        for repo_idx in tqdm(range(len(dataset[lang]))):
            for filepath in dataset[lang][repo_idx]["content"].keys():
                prev_byte_len = len(
                    bytes(dataset[lang][repo_idx]["content"][filepath], "utf-8")
                )
                dataset[lang][repo_idx]["content"][filepath] = remove_comments(
                    dataset[lang][repo_idx]["content"][filepath], lang
                )
                new_byte_len = len(
                    bytes(dataset[lang][repo_idx]["content"][filepath], "utf-8")
                )
                assert prev_byte_len == new_byte_len

    dataset_dir = "/".join(ds_filepath.split("/")[:-1])
    ds_filepath = ds_filepath.split("/")[-1]
    ds_fname = ".".join(ds_filepath.split(".")[:-1])
    ds_ext = ds_filepath.split(".")[-1]

    obfs_ds_file = open(
        os.path.join(dataset_dir, f"{ds_fname}-obfuscated.{ds_ext}"), "w+"
    )
    obfs_ds_file.write(json.dumps(dataset))
    obfs_ds_file.close()


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

from rich.progress import (
    BarColumn,
    MofNCompleteColumn,
    Progress,
    TextColumn,
    TimeElapsedColumn,
)

FUNCTION_QUERY = {
    "python": "(function_definition name: (_)) @fdef",
    "java": "(method_declaration name: (_)) @fdef",
    "typescript": "(function_declaration name: (_)) @fdef",
    "rust": "(function_item name: (_)) @fdef",
    "cpp": "(function_definition declarator: (function_declarator declarator: (identifier))) @fdef",
    "go": "(function_declaration name: (_)) @fdef",
}

COMMENT_QUERY = {
    "python": [
        "(block (expression_statement (string) @docstring))",
        "(comment) @comment",
    ],
    "java": ["(line_comment) @comment", "(block_comment) @comment"],
    "cpp": ["(comment) @comment"],
    "rust": ["(line_comment) @comment", "(block_comment) @comment"],
    "typescript": ["(comment) @comment"],
    "go": ["(comment) @comment"],
}

FUNCTION_NAME_QUERY = {
    "python": """
        ((function_definition
          name: (identifier) @function_name))
    """,
    "java": """
        (method_declaration
          name: (identifier) @method_name)
    """,
    "typescript": """
        (function_declaration
          name: (identifier) @function_name)
    """,
    "rust": """
        (function_item
          name: (identifier) @function_name)
    """,
    "cpp": """
        (function_definition
          name: (identifier) @function_name)
    """,
}


def topological_sort(graph):
    # Stack to store the topological order
    stack = []
    # Set to keep track of visited nodes
    visited = set()

    # Recursive function to process nodes
    def dfs(node):
        # Mark the current node as visited
        visited.add(node)
        # Recurse for all the vertices adjacent to this vertex
        for neighbour in graph.get(node, []):
            if neighbour not in visited:
                dfs(neighbour)
        # Push current vertex to stack which stores the result
        stack.append(node)

    # Call the recursive helper function to store the topological sort starting from all vertices one by one
    for node in graph:
        if node not in visited:
            dfs(node)

    return stack


def progress(note: str = "processing"):
    return Progress(
        TextColumn(f"{note} •" + "[progress.percentage]{task.percentage:>3.0f}%"),
        BarColumn(),
        MofNCompleteColumn(),
        TextColumn("•"),
        TimeElapsedColumn(),
    )



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import signal
import time

import google.generativeai as genai
from google.api_core.exceptions import GoogleAPICallError, ResourceExhausted

from repoqa.provider.request import construct_message_list


def make_request(
    client: genai.GenerativeModel,
    message: str,
    model: str,
    max_tokens: int = 512,
    temperature: float = 1,
    n: int = 1,
    system_msg="You are a helpful assistant good at coding.",
    **kwargs,
) -> genai.types.GenerateContentResponse:
    messages = []
    if system_msg:
        messages.append({"role": "system", "parts": [system_msg]})
    messages.append({"role": "user", "parts": [message]})
    return client.generate_content(
        messages,
        generation_config=genai.types.GenerationConfig(
            candidate_count=n, max_output_tokens=max_tokens, temperature=temperature
        ),
        **kwargs,
    )


def handler(signum, frame):
    # swallow signum and frame
    raise Exception("end of time")


def make_auto_request(*args, **kwargs) -> genai.types.GenerateContentResponse:
    ret = None
    while ret is None:
        try:
            signal.signal(signal.SIGALRM, handler)
            signal.alarm(100)
            ret = make_request(*args, **kwargs)
            signal.alarm(0)
        except ResourceExhausted as e:
            print("Rate limit exceeded. Waiting...", e.message)
            signal.alarm(0)
            time.sleep(10)
        except GoogleAPICallError as e:
            print(e.message)
            signal.alarm(0)
            time.sleep(1)
        except Exception as e:
            print("Unknown error. Waiting...")
            print(e)
            signal.alarm(0)
            time.sleep(1)
    return ret



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
from datetime import datetime

import git
import tempdir
from fire import Fire
from tqdm.auto import tqdm

from scripts.curate.utility import lang2suffix


def get_files_to_include(gh_repo, entrypoint, lang_suffix):
    files_to_include = []
    for entry in gh_repo.commit().tree.traverse():
        if entry.path.startswith(entrypoint) and any(
            [entry.path.endswith(suffix) for suffix in lang_suffix]
        ):
            files_to_include.append((entry.path, entry.abspath))
    return files_to_include


def main(
    target_path: str = f"repoqa-{datetime.now().isoformat()}.json",
):
    # read /scripts/cherrypick/lists.json
    with open("scripts/cherrypick/lists.json") as f:
        lists = json.load(f)

    for lang, repos in lists.items():
        lang_suffix = lang2suffix[lang]
        for repo in tqdm(repos):
            repo_name = repo["repo"]
            commit_sha = repo["commit_sha"]
            entrypoint = repo["entrypoint_path"]

            print(f"Visiting https://github.com/{repo_name}/tree/{commit_sha}")

            if repo.get("content"):
                print(f"Skipping {repo_name} as it already has content.")
                continue

            with tempdir.TempDir() as temp_dir:
                gh_repo = git.Repo.clone_from(
                    f"https://github.com/{repo_name}.git",
                    temp_dir,
                )
                gh_repo.git.checkout(commit_sha)

                files_to_include = get_files_to_include(
                    gh_repo, entrypoint, lang_suffix
                )

                repo["content"] = {}
                for path, abspath in files_to_include:
                    with open(abspath, "r") as f:
                        repo["content"][path] = f.read()
    with open(target_path, "w") as f_out:
        json.dump(lists, f_out)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import os
from typing import List

from anthropic import Client

from repoqa.provider.base import BaseProvider
from repoqa.provider.request.anthropic import make_auto_request


class AnthropicProvider(BaseProvider):
    def __init__(self, model):
        self.model = model
        self.client = Client(api_key=os.getenv("ANTHROPIC_KEY"))

    def generate_reply(
        self, question, n=1, max_tokens=1024, temperature=0.0, system_msg=None
    ) -> List[str]:
        assert temperature != 0 or n == 1, "n must be 1 when temperature is 0"
        replies = []
        for _ in range(n):
            reply = make_auto_request(
                self.client,
                message=question,
                model=self.model,
                temperature=temperature,
                max_tokens=max_tokens,
                system_msg=system_msg,
            )
            replies.append(reply.content[0].text)

        return replies



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
import os
from typing import List, Tuple

from transformers import AutoTokenizer

from repoqa.compute_score import compute_score, save_json
from repoqa.data import CACHE_DIR, get_repoqa_data
from repoqa.utility import progress, topological_sort

COMMENT_PREFIX = {
    "python": "#",
    "java": "//",
    "typescript": "//",
    "rust": "//",
    "cpp": "//",
}


# Model context below:
TEMPLATE = "instruction\ncode_context\ndescription\ninstruction"

INSTRUCTION = (
    "Based on the function description and code context,"
    " please retrieve and repeat the exact described function from the code context in a code block wrapped by ```:"
)


# Count number of tokens ignoring null byte (used for obfuscate-nl)
def count_tokens(tokens: List[str]) -> int:
    return len([x for x in tokens if x != "<0x00>"])


def _backward_tokenizable_lines(lines, tokenizer, max_tokens):
    """Return the text and tokens from bottom to top"""
    text = ""
    ntokens = 0
    is_break = False
    for line in reversed(lines):
        new_ntokens = count_tokens(tokenizer.tokenize(line + "\n"))
        if ntokens + new_ntokens > max_tokens:
            is_break = True
            break
        text = line + "\n" + text
        ntokens += new_ntokens
    return text, ntokens, is_break


def _forward_tokenizable_lines(lines, tokenizer, max_tokens):
    """Return the text and tokens from top to bottom"""
    text = ""
    ntokens = 0
    is_break = False
    for line in lines:
        new_ntokens = count_tokens(tokenizer.tokenize(line + "\n"))
        if ntokens + new_ntokens > max_tokens:
            is_break = True
            break
        text += line + "\n"
        ntokens += new_ntokens
    if is_break:
        text = text + "...\n"
        ntokens += count_tokens(tokenizer.tokenize("...\n"))
    return text, ntokens, is_break


def make_code_context(
    needle,
    file_content_list: List[Tuple[str, str]],
    position_ratio,
    code_context_size,
    language,
) -> str:
    """
    Slice the file_content_list such that:
    1. The slice contains code_context_size tokens
    2. The positon of the needle is at position_ratio of the slice*
    *May not be achievable if the needle is too close to the beginning or end of the file_content_list
    *May not be accurate as we will also insert file names at the beginning of each file
    *Token sizes might not be 100 accurate but should be close enough
    """
    tokenizer = AutoTokenizer.from_pretrained("codellama/CodeLlama-7b-Instruct-hf")

    needle_file_idx, needle_file_content = [
        (i, content)
        for i, (f, content) in enumerate(file_content_list)
        if f == needle["path"]
    ][0]

    needle_code = needle_file_content[needle["start_byte"] : needle["end_byte"]]
    ntoken_needle = count_tokens(tokenizer.tokenize(needle_code))

    prefix_size = int(code_context_size * position_ratio - ntoken_needle / 2)
    suffix_size = code_context_size - ntoken_needle - prefix_size

    # handling prefix of the needle file
    code_prefix, ntokens, is_break = _backward_tokenizable_lines(
        [COMMENT_PREFIX[language] + " Path: " + needle["path"]]
        + needle_file_content[: needle["start_byte"]].split("\n"),
        tokenizer,
        prefix_size,
    )
    prefix_size -= ntokens

    # handling prefix of the previous files
    index = needle_file_idx - 1
    while not is_break and prefix_size > 0 and index >= 0:
        path, content = file_content_list[index]
        prefix, ntokens, is_break = _forward_tokenizable_lines(
            [COMMENT_PREFIX[language] + " Path: " + path] + content.split("\n"),
            tokenizer,
            prefix_size,
        )
        code_prefix = prefix + code_prefix
        prefix_size -= ntokens
        index -= 1

    # handling suffix of the needle file
    code_suffix, ntokens, is_break = _forward_tokenizable_lines(
        needle_file_content[needle["end_byte"] :].split("\n"), tokenizer, suffix_size
    )
    suffix_size -= ntokens

    # handling suffix of the next files
    index = needle_file_idx + 1
    while not is_break and suffix_size > 0 and index < len(file_content_list):
        path, content = file_content_list[index]
        suffix, ntokens, is_break = _forward_tokenizable_lines(
            [COMMENT_PREFIX[language] + " Path: " + path] + content.split("\n"),
            tokenizer,
            suffix_size,
        )
        code_suffix += suffix
        suffix_size -= ntokens
        index += 1

    # Remove all null characters from all three portions (obfuscate-nl)
    code_prefix = code_prefix.replace("\0", "")
    needle_code = needle_code.replace("\0", "")
    code_suffix = code_suffix.replace("\0", "")

    code_context = code_prefix + needle_code + code_suffix

    needle_token_start = len(tokenizer.tokenize(code_prefix))
    needle_token_end = needle_token_start + len(tokenizer.tokenize(needle_code))
    code_context_ntokens = needle_token_end + len(tokenizer.tokenize(code_suffix))

    return {
        "code_context": code_context,
        "needle_token_start": needle_token_start,
        "needle_token_end": needle_token_end,
        "code_context_ntokens": code_context_ntokens,
    }


def make_task_id(lang, repo, needle_name):
    return f"{lang}::{repo}::{needle_name}"


def make_cache_id(lang, repo, needle_name, code_context_size, position_ratio):
    return f"{lang}::{repo}::{needle_name}::{code_context_size}::{position_ratio}"


def evaluate_model(
    model: str,
    base_url: str = None,
    backend: str = None,
    tensor_parallel_size: int = 1,
    code_context_size: int = 16 * 1024,
    max_new_tokens: int = 1024,
    result_dir: str = "results",
    languages: List[str] = None,
    caching: bool = True,  # if enabled, will cache the tasks which can be used to resume
    system_message: str = None,
    dataset_path: str = None,
    ignore_comments: bool = False,
    trust_remote_code: bool = False,
    attn_implementation=None,
):
    if backend is None:
        if base_url is not None:
            backend = "openai"
        else:
            backend = "vllm"
        print(f"Using {backend} as the backend")
    assert backend is not None, "Please specify the backend"

    if dataset_path is not None:
        with open(dataset_path) as f:
            dataset = json.load(f)
    else:
        dataset = get_repoqa_data()

    # makedir if not exists
    os.makedirs(result_dir, exist_ok=True)
    context_size_dir = os.path.join(result_dir, f"ntoken_{code_context_size}")
    os.makedirs(context_size_dir, exist_ok=True)
    model_output_path = os.path.join(
        context_size_dir,
        f"{model.replace('/', '_slash_')}.jsonl",
    )

    # resume from model_output_file
    if os.path.exists(model_output_path):
        with open(model_output_path) as f:
            model_outputs = [json.loads(line) for line in f]
    else:
        model_outputs = []

    # resume tasks from cache if any
    # schema: {"cache_id": .., **task}
    cache_file = os.path.join(CACHE_DIR, f"cache_ntoken_{code_context_size}_v1.jsonl")
    os.makedirs(CACHE_DIR, exist_ok=True)

    cache = {}
    if caching:
        print("🔥 Caching enabled")
        if os.path.exists(cache_file):
            with open(cache_file) as f:
                cache = [json.loads(line) for line in f]
                cache = {c["cache_id"]: c for c in cache}
                # remove the cache_id field in c
                for c in cache.values():
                    c.pop("cache_id")
            print(f"Resuming from cache: {cache_file} with {len(cache)} tasks")

    resumed_task_ids = {
        make_task_id(r["language"], r["repo"], r["name"]) for r in model_outputs
    }

    # for each task we include
    # "repo", "name", "language", "path",
    # "template", "position_ratio", "description", "instruction", "code_context"
    # "needle_token_start", "needle_token_end", "code_context_ntokens"
    tasks = []
    for lang, repos in dataset.items():
        if languages is not None and lang not in languages:
            print(f"Skipping {lang} as it is not selected; selected: {languages}")
            continue

        print(f"🔥 Preparing code context for {lang}...")
        with progress(f"Processing {lang} context") as pbar:
            for repo in pbar.track(repos):
                # skip if the repo does not have needles
                if "needles" not in repo:
                    pbar.console.print(
                        f"⚠️ Skipping {repo['repo']} ({lang}) as it does not have `needles` -- do needle analysis first"
                    )
                    continue

                ordered_paths = topological_sort(repo["dependency"])
                file_content_list = [
                    (path, repo["content"][path]) for path in ordered_paths
                ]
                for i, needle in enumerate(repo["needles"]):
                    task_id = make_task_id(lang, repo["repo"], needle["name"])
                    if task_id in resumed_task_ids:
                        pbar.console.print(
                            f"Skipping {task_id} as it is already in the results"
                        )
                        continue

                    position_ratio = (i + 0.5) / len(repo["needles"])
                    cache_id = make_cache_id(
                        lang,
                        repo["repo"],
                        needle["name"],
                        code_context_size,
                        position_ratio,
                    )
                    if cache_id in cache:
                        tasks.append(cache[cache_id])
                        continue

                    task = {
                        "repo": repo["repo"],
                        "name": needle["name"],
                        "language": lang,
                        "path": needle["path"],
                        "position_ratio": position_ratio,
                        "description": f"\nFunction Description:{needle['description']}\n",
                        "instruction": INSTRUCTION,
                        "template": TEMPLATE,
                    }
                    code_context_info = make_code_context(
                        needle,
                        file_content_list,
                        position_ratio=position_ratio,
                        code_context_size=code_context_size,
                        language=lang,
                    )
                    task.update(code_context_info)
                    tasks.append(task)

                    if caching:  # cache
                        with open(cache_file, "a") as f_out:
                            f_out.write(
                                json.dumps({"cache_id": cache_id, **task}) + "\n"
                            )

    # filter finished tasks again (in case a cache is used)
    tasks = [
        task
        for task in tasks
        if make_task_id(task["language"], task["repo"], task["name"])
        not in resumed_task_ids
    ]

    if len(tasks) == 0:
        print("No tasks to evaluate! Exiting...")
        return

    if backend == "openai":
        from repoqa.provider.openai import OpenAIProvider

        engine = OpenAIProvider(model, base_url=base_url)
    elif backend == "vllm":
        from repoqa.provider.vllm import VllmProvider

        engine = VllmProvider(
            model,
            tensor_parallel_size=tensor_parallel_size,
            max_model_len=int(code_context_size * 1.25),  # Magic number
            trust_remote_code=trust_remote_code,
        )
    elif backend == "anthropic":
        from repoqa.provider.anthropic import AnthropicProvider

        engine = AnthropicProvider(model)
    elif backend == "hf":
        from repoqa.provider.hf import HfProvider

        engine = HfProvider(
            model,
            trust_remote_code=trust_remote_code,
            attn_implementation=attn_implementation,
        )
    elif backend == "google":
        from repoqa.provider.google import GoogleProvider

        engine = GoogleProvider(model)
    else:
        raise ValueError(f"Unknown backend: {backend}")

    if not system_message:
        print("🔥 System message is disabled")
        system_message = None

    with open(model_output_path, "a") as f_out:
        with progress(f"Running {model}") as pbar:
            for task in pbar.track(tasks):
                actual_position_ratio = (
                    task["needle_token_start"] / task["code_context_ntokens"]
                )
                pbar.console.print(
                    f"Searching {task['name']} in {task['repo']} ({task['language']}) -- "
                    f"position ratio: actual={actual_position_ratio:.2f}, expected={task['position_ratio']}"
                )
                prompt = ""
                for key in task["template"].split("\n"):
                    prompt += task[key]

                replies = engine.generate_reply(
                    prompt, n=1, max_tokens=max_new_tokens, system_msg=system_message
                )
                result = {**task, "output": replies}
                f_out.write(json.dumps(result) + "\n")
                f_out.flush()
                model_outputs.append(result)

    file_base, _ = os.path.splitext(model_output_path)
    result_path = file_base + "-SCORES.json"
    output_json = compute_score(model, dataset, model_outputs, ignore_comments)
    save_json(output_json, result_path)


def main():
    from fire import Fire

    Fire(evaluate_model)


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import gzip
import json
import os

import tempdir
import wget
from appdirs import user_cache_dir

CACHE_DIR = user_cache_dir("repoqa")

REPOQA_DATA_OVERRIDE_PATH = os.getenv("REPOQA_DATA_OVERRIDE_PATH", None)
REPOQA_DATA_VERSION = os.getenv("REPOQA_DATA_VERSION", "2024-04-20")


def _get_repoqa_data_ready_path() -> str:
    if REPOQA_DATA_OVERRIDE_PATH:
        assert os.path.exists(
            REPOQA_DATA_OVERRIDE_PATH
        ), f"File not found: {REPOQA_DATA_OVERRIDE_PATH}"
        return REPOQA_DATA_OVERRIDE_PATH

    gzip_url = f"https://github.com/evalplus/repoqa_release/releases/download/{REPOQA_DATA_VERSION}/repoqa-{REPOQA_DATA_VERSION}.json.gz"
    cache_path = os.path.join(CACHE_DIR, f"repoqa-{REPOQA_DATA_VERSION}.json")
    # Check if human eval file exists in CACHE_DIR
    if not os.path.exists(cache_path):
        # Install HumanEval dataset and parse as json
        print(f"Downloading dataset from {gzip_url}")
        with tempdir.TempDir() as tmpdir:
            gzip_path = os.path.join(tmpdir, f"data.json.gz")
            wget.download(gzip_url, gzip_path)

            with gzip.open(gzip_path, "rb") as f:
                repoqa_data = f.read().decode("utf-8")

        # create CACHE_DIR if not exists
        os.makedirs(CACHE_DIR, exist_ok=True)
        # Write the original human eval file to CACHE_DIR
        with open(cache_path, "w") as f:
            f.write(repoqa_data)

    return cache_path


def get_repoqa_data():
    with open(_get_repoqa_data_ready_path(), "r") as f:
        return json.load(f)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

try:
    from repoqa._version import __version__, __version_tuple__
except ImportError:
    __version__ = "local-dev"



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json

from tqdm import tqdm
from tree_sitter_languages import get_language, get_parser

from repoqa.utility import COMMENT_QUERY, FUNCTION_QUERY, topological_sort

_default_name_parser = lambda node: node.child_by_field_name("name").text.decode()
_cpp_name_parser = (
    lambda node: node.child_by_field_name("declarator")
    .child_by_field_name("declarator")
    .text.decode()
)


def comment_analysis(code: bytes, language: str) -> float:
    query_texts = COMMENT_QUERY[language]
    parser = get_parser(language)
    tree = parser.parse(code)
    characters = 0
    for query_text in query_texts:
        comment_query = get_language(language).query(query_text)
        for node, _ in comment_query.captures(tree.root_node):
            comment_text = code[node.start_byte : node.end_byte]
            characters += len(comment_text)
    return characters / len(code)


# Annotate an incomplete repoqa dataset with function and class information
def main(dataset_path: str, overwrite_analysis: bool = False):
    assert dataset_path.endswith(".json"), "Dataset must be a JSON file, check README"
    with open(dataset_path, "r") as f:
        lists = json.load(f)

    for lang, repos in lists.items():
        assert (
            lang in FUNCTION_QUERY
        ), f"Unsupported language: {lang} -- supported: {FUNCTION_QUERY.keys()}"

        fn_query_text = FUNCTION_QUERY[lang]
        print(f"🔥 Querying {lang} functions with `{fn_query_text}`...")

        parser = get_parser(lang)
        fn_query = get_language(lang).query(fn_query_text)
        fn_name_parser = _cpp_name_parser if lang == "cpp" else _default_name_parser

        for repo in tqdm(repos):
            # skip if the repo already has function information
            if not overwrite_analysis and repo.get("functions"):
                continue

            if not repo.get("dependency"):
                print(
                    f"⚠️ Skipping {repo['repo']} ({lang}) as it does not have `dependency` -- do dependency analysis first"
                )
                continue

            ordered_paths = topological_sort(repo["dependency"])
            global_byte_idx = 0
            global_line_idx = 0
            functions = {}  # path to a list of functions
            for path in ordered_paths:
                code = repo["content"][path]
                code_bytes = bytes(code, "utf8")
                tree = parser.parse(code_bytes)
                extracted_functions = []
                for capture in fn_query.captures(tree.root_node):
                    node, _ = capture
                    function_content = code_bytes[node.start_byte : node.end_byte]
                    code_ratio = comment_analysis(function_content, lang)
                    extracted_functions.append(
                        {
                            "name": fn_name_parser(node),
                            "start_line": node.start_point[0],
                            "end_line": node.end_point[0] + 1,
                            "start_byte": node.start_byte,
                            "end_byte": node.end_byte,
                            "global_start_line": global_line_idx + node.start_point[0],
                            "global_end_line": global_line_idx + node.end_point[0] + 1,
                            "global_start_byte": global_byte_idx + node.start_byte,
                            "global_end_byte": global_byte_idx + node.end_byte,
                            "code_ratio": code_ratio,
                        }
                    )
                functions[path] = extracted_functions
                global_byte_idx += len(code)
                global_line_idx += code.count("\n") + 1
            repo["functions"] = functions
            print(
                f"🎉 Found {sum(len(v) for v in functions.values())} functions in {repo['repo']} ({lang})"
            )

    # update the dataset
    with open(dataset_path, "w") as f_out:
        json.dump(lists, f_out)


if __name__ == "__main__":
    from fire import Fire

    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json

from fire import Fire


def main(dataset_path: str, annotation_path: str):
    assert dataset_path.endswith(".json"), "Dataset must be a JSON file, check README"
    assert annotation_path.endswith(
        ".jsonl"
    ), "Annotation must be a JSONL file, check README"

    with open(dataset_path) as f:
        dataset = json.load(f)

    with open(annotation_path) as f:
        annotations = [json.loads(line) for line in f]

    def make_key(repo_name, func_name):
        return f"{repo_name}::{func_name}"

    key2annotation = {make_key(a["repo"], a["name"]): a for a in annotations}

    for lang, repos in dataset.items():
        for repo in repos:
            if "needles" not in repo:
                print(
                    f"⚠️ Skipping {repo['repo']} ({lang}) as it does not have `needles` -- do needle analysis first"
                )
                continue
            for needle in repo["needles"]:
                needle_name = needle["name"]
                key = make_key(repo["repo"], needle_name)
                annotation = key2annotation.get(key, None)
                if annotation is None:
                    print(
                        f"⚠️ Missing annotation for {key} for lang {lang} -- skipping"
                    )
                    continue
                needle["description"] = annotation["annotation"]

    with open(dataset_path, "w") as f_out:
        json.dump(dataset, f_out)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

lang2suffix = {
    "python": [".py"],
    "go": [".go"],
    "cpp": [".cpp", ".hpp", ".cc", ".hh", ".cxx", ".hxx", ".c", ".h"],
    "java": [".java"],
    "typescript": [".ts", ".js"],
    "php": [".php"],
    "rust": [".rs"],
}



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
import os
import subprocess
from pathlib import Path

import git
import tempdir
from fire import Fire
from tqdm.auto import tqdm

TOOL_JAR_PATH = (
    Path(__file__).resolve().parent
    / "java-analysis"
    / "java-lib"
    / "java-analysis-1.0-SNAPSHOT.jar"
)


CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))

# dataset_path is the dataset generated by dataset_ensemble_clone.py
def main():
    with open("scripts/cherrypick/lists.json") as f:
        lists = json.load(f)

    repos = lists["java"]
    for repo in tqdm(repos):
        repo_name = repo["repo"]
        commit_sha = repo["commit_sha"]
        entrypoint = repo["entrypoint_path"]
        print(f"Visiting https://github.com/{repo_name}/tree/{commit_sha}")

        with tempdir.TempDir() as temp_dir:
            gh_repo = git.Repo.clone_from(
                f"https://github.com/{repo_name}.git",
                temp_dir,
            )
            gh_repo.git.checkout(commit_sha)

            output_dependency = {}
            abs_prefix = os.path.join(temp_dir, entrypoint)
            for java_file in Path(abs_prefix).rglob("*.java"):
                command_list = f"java -jar {TOOL_JAR_PATH} {temp_dir} {entrypoint} {java_file.absolute()}".split()
                # cd `temp_dir`` and capture the output
                output = subprocess.check_output(command_list, cwd=temp_dir)
                dependencies = output.decode("utf-8").strip().split("\n")

                if dependencies:
                    if "" in dependencies:
                        dependencies.remove("")
                    output_dependency[
                        java_file.relative_to(Path(temp_dir)).as_posix()
                    ] = dependencies
                else:
                    output_dependency[
                        java_file.relative_to(Path(temp_dir)).as_posix()
                    ] = []

            repo["dependency"] = output_dependency

    with open(os.path.join(CURRENT_DIR, "data", "java.json"), "w") as f_out:
        json.dump({"java": repos}, f_out)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import re

from nltk.translate.bleu_score import SmoothingFunction, sentence_bleu


def compute_function_similarity(
    candidate_function: str, reference_function: str
) -> float:
    candidate_tokens = [item for item in re.split("\s+", candidate_function.strip())]

    reference_tokens = [item for item in re.split("\s+", reference_function.strip())]

    chencherry = SmoothingFunction()

    return sentence_bleu(
        [reference_tokens], candidate_tokens, smoothing_function=chencherry.method4
    )



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import signal
import time

import anthropic
from anthropic.types import Message

from repoqa.provider.request import construct_message_list


def make_request(
    client: anthropic.Client,
    message: str,
    model: str,
    max_tokens: int = 512,
    temperature: float = 1,
    system_msg="You are a helpful assistant good at coding.",
    **kwargs,
) -> Message:
    return client.messages.create(
        model=model,
        messages=construct_message_list(message, system_message=system_msg),
        max_tokens=max_tokens,
        temperature=temperature,
        **kwargs,
    )


def handler(signum, frame):
    # swallow signum and frame
    raise Exception("end of time")


def make_auto_request(client: anthropic.Client, *args, **kwargs) -> Message:
    ret = None
    while ret is None:
        try:
            signal.signal(signal.SIGALRM, handler)
            signal.alarm(100)
            ret = make_request(client, *args, **kwargs)
            signal.alarm(0)
        except anthropic.RateLimitError:
            print("Rate limit exceeded. Waiting...")
            signal.alarm(0)
            time.sleep(10)
        except anthropic.APIConnectionError:
            print("API connection error. Waiting...")
            signal.alarm(0)
            time.sleep(5)
        except anthropic.InternalServerError:
            print("Internal server error. Waiting...")
            signal.alarm(0)
            time.sleep(5)
        except anthropic.APIError as e:
            print("Unknown API error")
            print(e)
            if (
                e.body["error"]["message"]
                == "Output blocked by content filtering policy"
            ):
                raise Exception("Content filtering policy blocked output")
            signal.alarm(0)
        except Exception as e:
            print("Unknown error. Waiting...")
            print(e)
            signal.alarm(0)
            time.sleep(1)
    return ret



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

"""
! Note: not fully usable. You might encounter:
github.GithubException.GithubException: 422 {"message": "Validation Failed", "errors": [{"message": "The listed users and repositories cannot be searched either because the resources do not exist o
r you do not have permission to view them.", "resource": "Search", "field": "q", "code": "invalid"}], "documentation_url": "https://docs.github.com/v3/search/"}
"""

import json
import os
from datetime import datetime
from typing import TypedDict

from fire import Fire
from github import Auth, Github
from github.Repository import Repository
from tqdm.auto import tqdm

from scripts.curate.utility import lang2suffix


class GitHubRepoMeta(TypedDict):
    repo_name: str
    repo_owner: str
    commit_sha: str
    repo_size: int


class GitHubDocument(GitHubRepoMeta):
    timestamp: str
    path: str
    content: str


def main(
    target_path: str = f"repoqa-{datetime.now().isoformat()}.json",
):
    token = os.getenv("GITHUB_TOKEN")
    assert token is not None, "Make a token at https://github.com/settings/tokens"
    auth = Auth.Token(token)

    # read /scripts/cherrypick/lists.json
    with open("scripts/cherrypick/lists.json") as f:
        lists = json.load(f)

    g = Github(auth=auth, per_page=1)
    for lang, repos in lists.items():
        lang_suffix = lang2suffix[lang]
        for repo in tqdm(repos):
            if repo.get("content"):
                print(f"Skipping {repo['repo']} as it already has content.")
                continue

            repo_name = repo["repo"]
            commit_sha = repo["commit_sha"]
            entrypoint = repo["entrypoint_path"]
            query = f"repo:{repo_name}"

            gh_repos = g.search_repositories(query)
            gh_repo: Repository = gh_repos[0]
            contents = [
                item
                for item in gh_repo.get_contents(entrypoint, ref=commit_sha)
                if any([item.path.endswith(suffix) for suffix in lang_suffix])
            ]

            repo["content"] = {}
            for item in contents:
                if item.encoding != "base64":
                    continue
                file_content = item.decoded_content.decode("utf-8")
                repo["content"][item.path] = file_content

    with open(target_path, "w") as f_out:
        json.dump(lists, f_out)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

from typing import List

from transformers import AutoTokenizer
from vllm import LLM, SamplingParams

from repoqa.provider.base import BaseProvider
from repoqa.provider.request import construct_message_list, hacky_assistant_stop_seq


class VllmProvider(BaseProvider):
    def __init__(
        self, model, tensor_parallel_size, max_model_len=None, trust_remote_code=False
    ):
        self.tokenizer = AutoTokenizer.from_pretrained(
            model, trust_remote_code=trust_remote_code
        )
        self.llm = LLM(
            model=model,
            tensor_parallel_size=tensor_parallel_size,
            max_model_len=max_model_len,
            trust_remote_code=trust_remote_code,
        )
        self.stop_seq = []
        if self.tokenizer.chat_template:
            self.stop_seq.append(hacky_assistant_stop_seq(self.tokenizer))

    def generate_reply(
        self, question, n=1, max_tokens=1024, temperature=0.0, system_msg=None
    ) -> List[str]:
        assert temperature != 0 or n == 1, "n must be 1 when temperature is 0"

        prompt = self.tokenizer.apply_chat_template(
            construct_message_list(question, system_msg),
            tokenize=False,
            add_generation_prompt=True,
        )
        vllm_outputs = self.llm.generate(
            [prompt],
            SamplingParams(
                temperature=temperature,
                max_tokens=max_tokens,
                stop=self.stop_seq,
            ),
            use_tqdm=False,
        )

        gen_strs = [x.outputs[0].text for x in vllm_outputs]
        return gen_strs



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
import os
import shutil
import subprocess
from typing import Dict, List

import git
import tempdir
from fire import Fire
from tqdm.auto import tqdm

CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))


def remove_relative(path: str) -> str:
    parts = path.split(os.sep)
    filtered_parts = [part for part in parts if part not in (".", "..")]
    new_path = os.path.join(*filtered_parts)
    return new_path


def sanitize_paths(data: Dict[str, List[str]], entrypoint: str) -> Dict[str, List[str]]:
    sanitized_data = {}
    for file, dependencies in data.items():
        updated_file = os.path.join(entrypoint, remove_relative(file))
        updated_dependencies = []
        for dependency in dependencies:
            updated_dependency = os.path.join(entrypoint, remove_relative(dependency))
            updated_dependencies.append(updated_dependency)
        sanitized_data[updated_file] = updated_dependencies
    return sanitized_data


def run_dependency_analysis(config_file, go_file):
    # Load the JSON configuration
    with open(config_file, "r") as file:
        data = json.load(file)

    repos = data["go"]

    # Iterate over each repo entry in the JSON configuration
    for entry in tqdm(repos):
        repo_name = entry["repo"]
        commit_sha = entry["commit_sha"]
        entrypoint_path = entry["entrypoint_path"]

        print(f"Visiting https://github.com/{repo_name}/tree/{commit_sha}")

        with tempdir.TempDir() as temp_dir:
            gh_repo = git.Repo.clone_from(
                f"https://github.com/{repo_name}.git",
                temp_dir,
            )
            gh_repo.git.checkout(commit_sha)
            shutil.copy(go_file, temp_dir)

            command_list = (
                f"go build -o dependency_analysis dependency_analysis.go".split()
            )
            subprocess.run(command_list, cwd=temp_dir)

            command_list = f"./dependency_analysis {entrypoint_path} {repo_name.split('/')[-1]}".split()

            subprocess.run(command_list, cwd=temp_dir)
            output_dir = os.path.join(temp_dir, "output.json")
            with open(output_dir, "r") as output_file:
                output_data = json.load(output_file)
                entry["dependency"] = sanitize_paths(
                    output_data["repoName"], entrypoint_path
                )

    # Write all output data to a file
    with open(os.path.join(CURRENT_DIR, "data", "go.json"), "w") as f_out:
        json.dump({"go": repos}, f_out)


def main():
    config_file = "scripts/cherrypick/lists.json"
    go_file = "scripts/curate/dep_analysis/go-analysis/dependency_analysis.go"

    run_dependency_analysis(config_file, go_file)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0
import json
import os
import subprocess
from enum import Enum
from pathlib import Path
from typing import List, Tuple

import git
import pygraphviz as pgv
import tempdir
from fire import Fire
from pygraphviz import Node
from tqdm.auto import tqdm

CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))


class CrateType(Enum):
    LIB = ("lib",)
    BIN = "bin"


# Given a namespace, find its location in the file directory
def find_path(node: Node, temp_dir: str, entrypoint: str, type: CrateType) -> str:
    module_path = node.get_name()
    temp_dir = Path(temp_dir)
    entrypoint = Path(entrypoint)
    paths = module_path.split("::")

    current_path = entrypoint
    best_path = None

    for path in paths[1:]:
        current_path = current_path / path
        if (temp_dir / current_path / "mod.rs").exists():
            best_path = current_path / "mod.rs"
            continue
        elif (temp_dir / current_path).with_suffix(".rs").exists():
            best_path = current_path.with_suffix(".rs")
        break

    if not (best_path):
        if type == CrateType.LIB:
            return os.path.join(entrypoint, "lib.rs")
        elif type == CrateType.BIN:
            return os.path.join(entrypoint, "main.rs")
    return str(best_path)


# Find all buildable packages within the entrypoint
def find_packages(temp_dir: str, entrypoint: str) -> List[Tuple[str, str, CrateType]]:
    command_list = f"cargo metadata -q --no-deps --format-version=1".split()
    output = subprocess.check_output(command_list, cwd=temp_dir)
    decoded_output = output.decode("utf-8")

    result = json.loads(decoded_output)
    packages = result["packages"]
    results = []
    for package in packages:
        for target in package["targets"]:
            if not (os.path.join(temp_dir, entrypoint) in target["src_path"]):
                continue
            if target["kind"][0] == "lib":
                results.append((target["name"], target["src_path"], CrateType.LIB))
            elif target["kind"][0] == "bin":
                results.append((target["name"], target["src_path"], CrateType.BIN))
    return results


def is_cargo_modules_available() -> bool:
    try:
        command_list = ["cargo-modules", "--help"]
        subprocess.run(command_list, capture_output=True)
        return True
    except (subprocess.CalledProcessError, FileNotFoundError):
        return False


# dataset_path is the dataset generated by dataset_ensemble_clone.py
def main():
    if not (is_cargo_modules_available()):
        print("cargo-modules tool not found, exiting...")
        return

    with open("scripts/cherrypick/lists.json") as f:
        lists = json.load(f)

    repos = lists["rust"]
    for repo in tqdm(repos):
        repo_name = repo["repo"]
        commit_sha = repo["commit_sha"]
        entrypoint = repo["entrypoint_path"]

        print(f"Visiting https://github.com/{repo_name}/tree/{commit_sha}")

        with tempdir.TempDir() as temp_dir:
            gh_repo = git.Repo.clone_from(
                f"https://github.com/{repo_name}.git",
                temp_dir,
            )
            gh_repo.git.checkout(commit_sha)
            packages = find_packages(temp_dir, entrypoint)
            mapping = {}
            dependency = {}
            edges = []
            for package_name, src_path, crate_type in packages:
                if "_" in package_name:
                    package_name = "-".join(package_name.split("_"))
                analysis_param = ""

                # Run cargo-modules tool
                if crate_type == CrateType.BIN:
                    analysis_param = f"--bin {package_name}"
                else:
                    analysis_param = f"--lib"
                command_list = f"cargo modules dependencies --package {package_name} {analysis_param} --cfg-test --no-sysroot --no-traits --no-types".split()
                entrypoint = os.path.dirname(os.path.relpath(src_path, temp_dir))

                # cd `temp_dir`` and capture the output json
                output = subprocess.check_output(command_list, cwd=temp_dir)
                decoded_output = output.decode("utf-8")

                # Parse graph and get mapping
                graph = pgv.AGraph(string=decoded_output)
                for node in graph.nodes():
                    node_name = node.get_name()
                    mapping[node_name] = find_path(
                        node, temp_dir, entrypoint, crate_type
                    )
                    if mapping[node_name] in dependency:
                        continue
                    dependency[mapping[node_name]] = set()

                # Save edges for later
                for start_node, end_node in graph.edges():
                    edges.append((start_node.get_name(), end_node.get_name()))

            # Parse every edge for dependencies
            for start_name, end_name in edges:
                if (
                    not (start_name in mapping and end_name in mapping)
                    or mapping[start_name] == mapping[end_name]
                ):
                    continue
                dependency[mapping[start_name]].add(mapping[end_name])

            for key in dependency:
                dependency[key] = list(dependency[key])

            repo["dependency"] = dependency

    with open(os.path.join(CURRENT_DIR, "data", "rust.json"), "w") as f_out:
        json.dump({"rust": repos}, f_out)


if __name__ == "__main__":
    Fire(main)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

# Estimate the maximum token size for a repository

import json

import matplotlib.pyplot as plt
from transformers import AutoTokenizer

from repoqa.utility import topological_sort

COMMENT_PREFIX = {
    "python": "#",
    "java": "//",
    "typescript": "//",
    "rust": "//",
    "cpp": "//",
}


def get_max_token_size(dataset_path: str, model: str):
    with open(dataset_path, "r") as f:
        repo = json.load(f)

    tokenizer = AutoTokenizer.from_pretrained(model)

    min_token_size = 1e9
    min_token_size_repo = None
    token_sizes = []

    for lang, repos in repo.items():
        for repo in repos:
            if "dependency" not in repo:
                print(
                    f"⚠️ Skipping {repo['repo']} ({lang}) as it does not have `dependency` -- do dependency analysis first"
                )
                continue

            ordered_paths = topological_sort(repo["dependency"])

            bigfile = ""
            for path in ordered_paths:
                bigfile += (
                    COMMENT_PREFIX[lang]
                    + f" current path: {path}\n"
                    + repo["content"][path]
                )

            # estimate the maximum token size
            token_size = tokenizer(bigfile, return_tensors="pt")["input_ids"].shape[1]
            token_sizes.append(token_size)
            min_token_size = min(min_token_size, token_size)
            if min_token_size == token_size:
                min_token_size_repo = repo["repo"]
            print(f"[{lang}] {repo['repo']:<32}: {token_size:>20} tokens")

    print(f"Estimated minimum token size: {min_token_size} by {min_token_size_repo}")

    # visualize the distribution
    plt.figure(figsize=(8, 4))
    plt.hist(token_sizes, bins=64)
    # xtick at every 20k
    unit = 100 * 1000
    plt.xticks(range(0, max(token_sizes) + 1, unit))
    plt.xlim(0, max(token_sizes) + 1000)
    # xtick using "k"
    plt.gca().xaxis.set_major_formatter(
        plt.FuncFormatter(lambda x, _: f"{x / unit:.0f}")
    )
    plt.xlabel("Token size (100k)")
    plt.ylabel("Frequency")
    plt.title("Token size distribution")
    # compact layout
    plt.tight_layout()
    plt.savefig("token_size_distribution.png", dpi=164, bbox_inches="tight")


if __name__ == "__main__":
    from fire import Fire

    Fire(get_max_token_size)



# SPDX-FileCopyrightText: (c) 2024 EvalPlus Team
#
# SPDX-License-Identifier: Apache-2.0

import json
from collections import Counter
from random import sample

from tqdm import tqdm


# Annotate an incomplete repoqa dataset with function and class information
def main(
    dataset_path: str,
    overwrite_analysis: bool = False,
    max_len: int = 2000,
    num_bins: int = 64,
    max_fn_per_repo: int = 10,
):
    assert (
        num_bins >= max_fn_per_repo
    ), "Number of bins must be greater than max functions per repo"
    assert dataset_path.endswith(".json"), "Dataset must be a JSON file, check README"
    with open(dataset_path, "r") as f:
        lists = json.load(f)

    for lang, repos in lists.items():
        print(f"🔥 Selecting needle functions for {lang}")
        # FIXME(@ganler): enable more dependency analysis!
        for repo in tqdm(repos):
            # skip if the repo already has function information
            if not overwrite_analysis and repo.get("needles"):
                continue

            if not repo.get("functions"):
                print(
                    f"⚠️ Skipping {repo['repo']} ({lang}) as it does not have `functions` field -- do function analysis first"
                )
                continue

            repo_size_bytes = sum(len(content) for content in repo["content"].values())

            selected_bins = set()
            bin_size = repo_size_bytes // num_bins

            function_names = Counter()
            for funcs in repo["functions"].values():
                for fn in funcs:
                    function_names.update([fn["name"]])
            # get function names that only appear once
            function_names = {k for k, v in function_names.items() if v == 1}

            needle_candidates = []
            for path, funcs in repo["functions"].items():
                for fn in funcs:
                    # criteria 1: no repeated function names
                    if fn["name"] not in function_names:
                        continue

                    # criteria 2: length <= max_len
                    if fn["end_byte"] - fn["start_byte"] > max_len:
                        continue

                    # criteria 3: not in the same bin
                    bin_idx = fn["global_start_byte"] // bin_size
                    if bin_idx in selected_bins:
                        continue

                    # criteria 4: TODO -- select those with more code!
                    selected_bins |= {bin_idx}
                    needle_candidates.append((path, fn))

            len_total_fn = sum(len(v) for v in repo["functions"].values())
            print(
                f"🎉 Selected {len(needle_candidates)} needles from {len_total_fn} functions in {repo['repo']} ({lang})"
            )
            needles = []
            for path, fn in sample(
                needle_candidates, min(max_fn_per_repo, len(needle_candidates))
            ):
                needles.append({**fn, "path": path})
            repo["needles"] = needles

    # update the dataset
    with open(dataset_path, "w") as f_out:
        json.dump(lists, f_out)


if __name__ == "__main__":
    from fire import Fire

    Fire(main)
